// SPDX-FileCopyrightText: Copyright © DUNE Project contributors, see file AUTHORS.md
// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception OR LGPL-3.0-or-later

#ifndef DUNE_ASSEMBLER_BACKENDS_FLATSET_HH
#define DUNE_ASSEMBLER_BACKENDS_FLATSET_HH

#include <algorithm>
#include <vector>

namespace Dune::Assembler {

template <class T>
class FlatSet
  : public std::vector<T>
{
  using Base = std::vector<T>;

public:
  using value_type = typename Base::value_type;

public:
  using Base::Base;
  using Base::begin;
  using Base::end;

  void insert (const value_type& value)
  {
    auto it = std::lower_bound(begin(), end(), value);
    if ((it == end() or (*it != value)))
      Base::insert(it, value);
  }

  [[nodiscard]] bool contains (const value_type& value) const
  {
    return std::binary_search(begin(), end(), value);
  }
};

} // end namespace Dune::Assembler

#endif // DUNE_ASSEMBLER_BACKENDS_FLATSET_HH
