// SPDX-FileCopyrightText: Copyright © DUNE Project contributors, see file AUTHORS.md
// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception OR LGPL-3.0-or-later

#ifndef DUNE_ASSEMBLER_BACKENDS_FLATVECTOR_HH
#define DUNE_ASSEMBLER_BACKENDS_FLATVECTOR_HH

#include <vector>

#include <dune/common/std/layout_right.hh>
#include <dune/common/std/mdarray.hh>

namespace Dune::Assembler {

template <class T>
class FlatVector
  : public Std::mdarray<T, Std::dextents<std::size_t,1>, Std::layout_right, std::vector<T>>
{
  using Base = Std::mdarray<T, Std::dextents<std::size_t,1>, Std::layout_right, std::vector<T>>;
  using index_type = typename Base::index_type;
  using value_type = typename Base::value_type;
  using extents_type = typename Base::extents_type;
  using mapping_type = typename Base::mapping_type;

public:
  using Base::Base;

  constexpr FlatVector& operator= (const value_type& value)
  {
    for (index_type i = 0; i < size(); ++i)
      (*this)[i] = value;
    return *this;
  }

  constexpr index_type size () const noexcept { return Base::extent(0); }

  /// \name Modifiers
  /// @{

  /// \brief Change the shape of the matrix and resize the underlying container with given default value
  void resize (index_type size, const value_type& value)
  {
    if (size != this->size())
    {
      auto container = std::move(*this).extract_container();
      auto m = mapping_type{extents_type{size}};
      container.resize(m.required_span_size(), value);
      static_cast<Base&>(*this) = Base{m, std::move(container)};
    }
  }

  /// \brief Change the shape of the matrix and resize the underlying container
  void resize (index_type size)
  {
    if (size != this->size())
    {
      auto container = std::move(*this).extract_container();
      auto m = mapping_type{extents_type{size}};
      container.resize(m.required_span_size());
      static_cast<Base&>(*this) = Base{m, std::move(container)};
    }
  }

  /// @}
};

} // end namespace Dune::Assembler

#endif // DUNE_ASSEMBLER_BACKENDS_FLATVECTOR_HH
