// SPDX-FileCopyrightText: Copyright © DUNE Project contributors, see file AUTHORS.md
// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception OR LGPL-3.0-or-later

#ifndef DUNE_ASSEMBLER_BACKENDS_ISTLVECTORBACKEND_HH
#define DUNE_ASSEMBLER_BACKENDS_ISTLVECTORBACKEND_HH

#include <dune/assembler/backends/flatvector.hh>
#include <dune/functions/backends/istlvectorbackend.hh>

namespace Dune::Assembler {

template <class Vector>
class ISTLVectorBackend
  : public Dune::Functions::Impl::ISTLVectorBackend<Vector>
{
  using Base = Dune::Functions::Impl::ISTLVectorBackend<Vector>;

public:
  using value_type = typename Vector::field_type;
  using LocalVector = FlatVector<value_type>;

public:
  ISTLVectorBackend (Vector& vector)
    : Base(vector)
  {}

  template <class LocalView>
  void scatter (const LocalView& localView, const LocalVector& localVector)
  {
    for (std::size_t i = 0; i < localView.size(); ++i)
      (*this)[localView.index(i)] += localVector[i];
  }

  void setZero ()
  {
    Base::operator= (0);
  }

  using Base::operator=;
};

} // end namespace Dune::Assembler

#endif // DUNE_ASSEMBLER_BACKENDS_ISTLVECTORBACKEND_HH
