// SPDX-FileCopyrightText: Copyright © DUNE Project contributors, see file AUTHORS.md
// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception OR LGPL-3.0-or-later

#ifndef DUNE_ASSEMBLER_CONCEPTS_HH
#define DUNE_ASSEMBLER_CONCEPTS_HH

#include <dune/common/concept.hh>

#include <dune/functions/functionspacebases/concepts.hh>

namespace Dune::Assembler::Concepts {

template <class LV>
concept LocalView =
  Dune::models<Functions::Concept::LocalView<typename LV::GlobalBasis>, LV>().value;

template <class GB>
concept GlobalBasis =
  Dune::models<Functions::Concept::GlobalBasis<typename GB::GridView>, GB>().value;


template <class M>
concept Matrix = requires(M matrix, int i, int j) {
  matrix[i][j];
};

template <class M>
concept BlockMatrix = requires(M matrix, int i, int j) {
  { matrix[i][j] } -> Matrix;
};

template <class V>
concept Vector = requires(V vector, int i) {
  vector[i];
};

template <class V>
concept BlockVector = requires(V vector, int i) {
  { vector[i] } -> Vector;
};

template <class P>
concept LocalMatrixPattern = requires(P pattern, int i, int j) {
  pattern.add(i, j);
  pattern.addAll();
  pattern.clear();
};

template <class P>
concept BlockLocalMatrixPattern = requires(P pattern, int i, int j) {
  { pattern[i][j] } -> LocalMatrixPattern;
};

namespace Archetypes {

struct Vector
{
  void operator[] (int) {}
};

struct Matrix
{
  auto operator[] (int) { return Vector{}; }
};

struct LocalMatrixPattern
{
  void add (int,int) {}
  void addAll () {}
  void clear () {}
};

} // end namespace Archetypes

static_assert(Concepts::Matrix<Archetypes::Matrix>);
static_assert(Concepts::Vector<Archetypes::Vector>);
static_assert(Concepts::LocalMatrixPattern<Archetypes::LocalMatrixPattern>);

} // end namespace Dune::Assembler::Concept

#endif // DUNE_ASSEMBLER_CONCEPTS_HH
