// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:

// SPDX-FileCopyrightText: Copyright © DUNE Project contributors, see file AUTHORS.md
// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception OR LGPL-3.0-or-later

#include <config.h>

#include <dune/assembler/assemblerconcepts.hh>
#include <dune/assembler/archetypes/localelementmatrixassembler.hh>
#include <dune/assembler/archetypes/localboundaryintersectionmatrixassembler.hh>
#include <dune/assembler/archetypes/localinteriorintersectionmatrixassembler.hh>

#include <dune/common/parallel/mpihelper.hh>
#include <dune/functions/functionspacebases/lagrangebasis.hh>
#include <dune/grid/yaspgrid.hh>


int main (int argc, char** argv)
{
  using namespace Dune::Assembler;

  Dune::MPIHelper::instance(argc, argv);

  auto grid = Dune::YaspGrid<2>({1.0,1.0}, {2,2});
  auto gridView = grid.leafGridView();

  using namespace Dune::Functions::BasisFactory;
  auto testBasis = makeBasis(gridView, lagrange<1>());
  using TestBasis = decltype(testBasis);
  static_assert(Concepts::GlobalBasis<TestBasis>);
  using TestLocalView = typename TestBasis::LocalView;
  static_assert(Concepts::LocalView<TestLocalView>);

  auto trialBasis = makeBasis(gridView, lagrange(1));
  using TrialBasis = decltype(trialBasis);
  static_assert(Concepts::GlobalBasis<TrialBasis>);
  using TrialLocalView = typename TrialBasis::LocalView;
  static_assert(Concepts::LocalView<TrialLocalView>);

  { // check the assembler concept with archetypes

    using ElementAssembler = Concepts::Archetypes::LocalElementMatrixAssembler<TestLocalView, TrialLocalView>;
    static_assert(Concepts::LocalElementMatrixAssembler<ElementAssembler, TestLocalView, TrialLocalView>);

    using BoundaryIntersectionAssembler = Concepts::Archetypes::LocalBoundaryIntersectionMatrixAssembler<TestLocalView, TrialLocalView>;
    static_assert(Concepts::LocalBoundaryIntersectionMatrixAssembler<BoundaryIntersectionAssembler, TestLocalView, TrialLocalView>);

    using InteriorIntersectionAssembler = Concepts::Archetypes::LocalInteriorIntersectionMatrixAssembler<TestLocalView, TrialLocalView>;
    static_assert(Concepts::LocalInteriorIntersectionMatrixAssembler<InteriorIntersectionAssembler, TestLocalView, TrialLocalView>);
  }
}
