// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:

// SPDX-FileCopyrightText: Copyright © DUNE Project contributors, see file AUTHORS.md
// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception OR LGPL-3.0-or-later

#include <config.h>
#include <iostream>

#include <dune/assembler/defaultglobalassembler.hh>
#include <dune/assembler/backends/istlmatrixbackend.hh>

#include <dune/common/parallel/mpihelper.hh>
#include <dune/functions/functionspacebases/lagrangebasis.hh>
#include <dune/grid/yaspgrid.hh>
#include <dune/istl/bcrsmatrix.hh>
#include <dune/istl/io.hh>

#include "massmatrixlocalassembler.hh"

int main (int argc, char** argv)
{
  Dune::MPIHelper::instance(argc, argv);

  auto grid = Dune::YaspGrid<2>({1.0,1.0}, {2,2});
  auto gridView = grid.leafGridView();

  using namespace Dune::Functions::BasisFactory;
  auto basis = makeBasis(gridView, lagrange<1>());
  using Basis = decltype(basis);

  auto matrix = Dune::BCRSMatrix<double>{};
  auto matrixBackend = Dune::Assembler::ISTLMatrixBackend{matrix};
  auto assembler = Dune::Assembler::Assembler{basis,basis};

  using LocalView = typename Basis::LocalView;
  auto localAssembler = Dune::Assembler::MassMatrixLocalAssembler<LocalView,LocalView>{};

  auto matrixPattern = matrixBackend.patternBuilder();
  matrixPattern.resize(basis, basis);
  assembler.assembleMatrixPattern(localAssembler, matrixPattern);
  matrixPattern.setupMatrix();
  matrix = 0;

  assembler.assembleMatrixEntries(localAssembler, matrixBackend);
  Dune::printmatrix(std::cout, matrix, "M", "[");
}
